@@ -51,6 +51,25 @@ class AdminApi:
         self.w3.eth.sign(addr, text='666f6f')
 
 
+    def account(self, address, cols=['tx_hash', 'sender', 'recipient', 'nonce', 'block', 'tx_index', 'status', 'network_status']):
+        s = celery.signature(
+                'cic_eth.queue.tx.get_account_tx',
+                [address],
+                queue=self.queue,
+                )
+        txs = s.apply_async().get()
+
+        tx_dict_list = []
+        for tx_hash in txs.keys():
+            tx = self.tx(tx_hash)
+            tx_cache = {}
+            for k in cols:
+                tx_cache[k] = tx[k]
+            tx_dict_list.append(tx_cache)
+
+        return tx_dict_list
+
+
     def tx(self, tx_hash=None, tx_raw=None):
 
         if tx_hash != None and tx_raw != None:
@@ -68,12 +87,22 @@ class AdminApi:
         tx = s.apply_async().get()
   
         source_token = None
+        if tx['source_token'] != zero_address:
+            try:
+                source_token = CICRegistry.get_address(CICRegistry.chain_spec, tx['source_token']).contract
+            except UnknownContractError:
+                source_token_contract = self.w3.eth.contract(abi=CICRegistry.abi('ERC20'), address=tx['source_token'])
+                source_token = CICRegistry.add_token(CICRegistry.chain_spec, source_token_contract)
+                logg.warning('unknown source token contract {}'.format(tx['source_token']))
+
         destination_token = None
-        try:
-            source_token = CICRegistry.get_address(CICRegistry.chain_spec, tx['source_token'])
-            destination_token = CICRegistry.get_address(CICRegistry.chain_spec, tx['desination_token'])
-        except UnknownContractError:
-            pass
+        if tx['source_token'] != zero_address:
+            try:
+                destination_token = CICRegistry.get_address(CICRegistry.chain_spec, tx['destination_token'])
+            except UnknownContractError:
+                destination_token_contract = self.w3.eth.contract(abi=CICRegistry.abi('ERC20'), address=tx['source_token'])
+                destination_token = CICRegistry.add_token(CICRegistry.chain_spec, destination_token_contract)
+                logg.warning('unknown destination token contract {}'.format(tx['destination_token']))
 
         tx['sender_description'] = 'Custodial account'
         tx['recipient_description'] = 'Custodial account'
@@ -116,9 +145,11 @@ class AdminApi:
 
         if source_token != None:
             tx['source_token_symbol'] = source_token.symbol()
+            tx['sender_token_balance'] = source_token.function('balanceOf')(tx['sender']).call()
 
         if destination_token != None:
             tx['destination_token_symbol'] = destination_token.symbol()
+            tx['recipient_token_balance'] = source_token.function('balanceOf')(tx['recipient']).call()
 
         tx['network_status'] = 'Not submitted'
 
