@@ -8,6 +8,8 @@ import logging
 
 # third-party imports 
 import celery
+from cic_registry.chain import ChainSpec
+from cic_eth.eth.factory import TxFactory
 
 app = celery.current_app
 
@@ -22,25 +24,29 @@ class Api:
     :param callback_param: URL to HTTP POST result data, optional
     :type callback_param: str
     :param callback_task: Callback task that executes callback_param call. (Must be included by the celery worker)
-    :type callback_task: function
+    :type callback_task: string
     :param queue: Name of worker queue to submit tasks to
     :type queue: str
     """
-    def __init__(self, callback_param=None, callback_task='cic_eth_callbacks.noop', queue='cic-eth'):
+    def __init__(self, queue='cic-eth', callback_param=None, callback_task='cic_eth.callbacks.noop', callback_queue=None, chain_str='Bloxberg:8995'):
+        self.chain_spec = ChainSpec.from_chain_str(chain_str)
         self.callback_param = callback_param
         self.callback_task = callback_task
         self.queue = queue
         logg.info('api using queue {}'.format(self.queue))
         self.callback_success = None
         self.callback_error = None
+        if callback_queue == None:
+            callback_queue=self.queue
 
         if callback_param != None:
             self.callback_success = celery.signature(
                     callback_task,
                     [
                         callback_param,
+                        0,
                         ],
-                    queue=queue,
+                    queue=callback_queue,
                     )
             self.callback_error = celery.signature(
                     callback_task,
@@ -48,9 +54,10 @@ class Api:
                         callback_param,
                         1,
                         ],
-                    queue=queue,
+                    queue=callback_queue,
                     )       
-
+        
+        TxFactory.init(None, self.chain_spec)
 
     def convert_transfer(self, from_address, to_address, target_return, minimum_return, from_token_symbol, to_token_symbol):
         """Executes a chain of celery tasks that performs conversion between two ERC20 tokens, and transfers to a specified receipient after convert has completed.
