@@ -39,7 +39,7 @@ class AccountTxFactory(TxFactory):
             address,
             ):
 
-        chain_spec = CICRegistry.accounts_chain_spec
+        chain_spec = AccountTxFactory.chain_spec
         c = CICRegistry.get_contract(chain_spec, 'AccountRegistry')
         f = c.function('add')
         tx_add_buildable = f(
@@ -60,7 +60,7 @@ class AccountTxFactory(TxFactory):
         self,
         address,
         ):
-        chain_spec = CICRegistry.accounts_chain_spec
+        chain_spec = AccountTxFactory.chain_spec
         c = CICRegistry.get_contract(chain_spec, 'Faucet')
         f = c.function('giveTo')
         tx_add_buildable = f(address)
@@ -136,7 +136,7 @@ def register(self, account_address, writer_address):
     queue = self.request.delivery_info['routing_key']
 
     c = RpcClient(writer_address)
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = AccountTxFactory.chain_spec
     AccountTxFactory.init(c.w3, chain_spec)
     txf = AccountTxFactory(writer_address)
 
@@ -176,7 +176,7 @@ def gift(self, account_address):
     queue = self.request.delivery_info['routing_key']
 
     c = RpcClient(account_address)
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = AccountTxFactory.chain_spec
     AccountTxFactory.init(c.w3, chain_spec)
     txf = AccountTxFactory(account_address)
 
@@ -244,7 +244,7 @@ def cache_gift_data(
         ):
 
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, AccountTxFactory.chain_spec.chain_id())
     tx_data = unpack_gift(tx['data'])
 
     tx_cache = TxCache(
@@ -273,7 +273,7 @@ def cache_account_data(
         ):
 
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, AccountTxFactory.chain_spec.chain_id())
     tx_data = unpack_register(tx['data'])
 
     tx_cache = TxCache(
