@@ -5,9 +5,9 @@ import logging
 # third-party imports
 import celery
 import web3
-
-# platform imports
 from cic_registry import CICRegistry
+
+# local imports
 from cic_eth.db import SessionBase
 from cic_eth.db.models.convert import TxConvertTransfer
 from cic_eth.db.models.otx import Otx
@@ -88,7 +88,7 @@ class BancorTxFactory(TxFactory):
                 'from': self.address,
                 'gas': network_gas,
                 'gasPrice': self.gas_price,
-                'chainId': CICRegistry.chain_spec.chain_id(),
+                'chainId': BancorTxFactory.chain_spec.chain_id(),
                 'nonce': self.next_nonce(),
                 })
         return tx_convert
@@ -158,7 +158,7 @@ def convert_with_default_reserve(self, tokens, from_address, source_amount, mini
 
     c = RpcClient(from_address)
 
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = BancorTxFactory.chain_spec
 
     cr = CICRegistry.get_contract(chain_spec, 'BancorNetwork')
     source_token = CICRegistry.get_address(chain_spec, tokens[0]['address'])
@@ -286,7 +286,7 @@ def resolve_converters_by_tokens(tokens):
     :return: Addresses of matching converters
     :rtype: list of str, 0x-hex
     """
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = BancorTxFactory.chain_spec
     for t in tokens:
         c = CICRegistry.get_contract(chain_spec, 'ConverterRegistry')
         fn = c.function('getConvertersByAnchors')
@@ -323,7 +323,7 @@ def transfer_converted(self, tokens, holder_address, receiver_address, value, tx
 
     queue = self.request.delivery_info['routing_key']
 
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = BancorTxFactory.chain_spec
     c = RpcClient(holder_address)
 
     # get transaction parameters
@@ -367,7 +367,7 @@ def otx_cache_convert(
         tx_signed_raw_hex,
         ):
 
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = BancorTxFactory.chain_spec
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
     tx = unpack_signed_raw_tx(tx_signed_raw_bytes, chain_spec.chain_id())
     tx_data = unpack_convert(tx['data'])
