@@ -21,19 +21,19 @@ celery_app = celery.current_app
 logg = logging.getLogger()
 
 contract_function_signatures = {
-        'request': '6c355091',
+        'request': 'b0addede',
         }
 
 
 class TransferRequestTxFactory(TxFactory):
     """Factory for creating ERC20 token transactions.
     """
-    def request(self, token_address, spender_address, amount):
-        chain_spec = CICRegistry.chain_spec
+    def request(self, token_address, beneficiary_address, amount):
+        chain_spec = TransferRequestTxFactory.chain_spec
         transfer_approval = CICRegistry.get_contract(chain_spec, 'TransferApproval')
-        fn = transfer_approval.function('request')
-        tx_approval_buildable = fn(spender_address, token_address, amount)
-        transfer_approval_gas = transfer_approval.gas('request')
+        fn = transfer_approval.function('createRequest')
+        tx_approval_buildable = fn(beneficiary_address, token_address, amount)
+        transfer_approval_gas = transfer_approval.gas('createRequest')
 
         tx_approval = tx_approval_buildable.buildTransaction({
             'from': self.address,
@@ -71,8 +71,8 @@ def transfer_approval_request(self, tokens, holder_address, receiver_address, va
 
     c = RpcClient(holder_address)
 
-    chain_spec = CICRegistry.chain_spec
-    TransferRequestTxFactory.init(c.w3, chain_spec) #, holder_address)
+    #chain_spec = .chain_spec
+    #TransferRequestTxFactory.init(c.w3, chain_spec) #, holder_address)
     txf = TransferRequestTxFactory(holder_address)
 
     tx_transfer = txf.request(t['address'], receiver_address, value)
@@ -98,7 +98,7 @@ def otx_cache_transfer_approval_request(
         ):
 
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, TransferRequestTxFactory.chain_spec.chain_id())
     (txc, cache_id) = cache_transfer_approval_request_data(tx_hash_hex, tx, True)
     return txc
 
