@@ -28,6 +28,7 @@ logg = logging.getLogger()
 contract_function_signatures = {
         'transfer': 'a9059cbb',
         'approve': '095ea7b3',
+        'transferfrom': '23b872dd',
         }
 
 
@@ -50,8 +51,7 @@ class TokenTxFactory(TxFactory):
         :returns: Unsigned "approve" transaction in standard Ethereum format
         :rtype: dict
         """
-        chain_spec = CICRegistry.chain_spec
-        source_token = CICRegistry.get_address(chain_spec, token_address)
+        source_token = CICRegistry.get_address(TokenTxFactory.chain_spec, token_address)
         source_token_contract = source_token.contract
         tx_approve_buildable = source_token_contract.functions.approve(
             spender_address,
@@ -63,7 +63,7 @@ class TokenTxFactory(TxFactory):
             'from': self.address,
             'gas': source_token_gas,
             'gasPrice': self.gas_price,
-            'chainId': chain_spec.chain_id(),
+            'chainId': self.chain_id,
             'nonce': self.next_nonce(),
             })
         return tx_approve
@@ -86,8 +86,7 @@ class TokenTxFactory(TxFactory):
         :returns: Unsigned "transfer" transaction in standard Ethereum format
         :rtype: dict
         """
-        chain_spec = CICRegistry.chain_spec
-        source_token = CICRegistry.get_address(chain_spec, token_address)
+        source_token = CICRegistry.get_address(TokenTxFactory.chain_spec, token_address)
         source_token_contract = source_token.contract
         transfer_buildable = source_token_contract.functions.transfer(
                 receiver_address,
@@ -118,6 +117,19 @@ def unpack_transfer(data):
         }
 
 
+def unpack_transferfrom(data):
+    f = data[2:10]
+    if f != contract_function_signatures['transferfrom']:
+        raise ValueError('Invalid transferFrom data ({})'.format(f))
+
+    d = data[10:]
+    return {
+        'from': web3.Web3.toChecksumAddress('0x' + d[64-40:64]),
+        'to': web3.Web3.toChecksumAddress('0x' + d[128-40:128]),
+        'amount': int(d[128:], 16)
+        }
+
+
 def unpack_approve(data):
     f = data[2:10]
     if f != contract_function_signatures['approve']:
@@ -149,7 +161,7 @@ def balance(tokens, holder_address):
         #token = CICRegistry.get_address(t['address'])
         #abi = token.abi()
         #o = c.w3.eth.contract(abi=abi, address=t['address'])
-        o = CICRegistry.get_address(CICRegistry.chain_spec, t['address']).contract
+        o = CICRegistry.get_address(TokenTxFactory.chain_spec, t['address']).contract
         b = o.functions.balanceOf(holder_address).call()
         logg.debug('balance {} for {}: {}'.format(t['address'], holder_address, b))
         balances.append(b)
@@ -186,7 +198,7 @@ def transfer(self, tokens, holder_address, receiver_address, value):
 
     c = RpcClient(holder_address)
 
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = TokenTxFactory.chain_spec
     TokenTxFactory.init(c.w3, chain_spec) #, holder_address)
     txf = TokenTxFactory(holder_address)
 
@@ -236,7 +248,7 @@ def approve(self, tokens, holder_address, spender_address, value):
 
     c = RpcClient(holder_address)
 
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = TokenTxFactory.chain_spec
     TokenTxFactory.init(c.w3, chain_spec) #, holder_address)
     txf = TokenTxFactory(holder_address)
 
@@ -267,7 +279,7 @@ def resolve_tokens_by_symbol(token_symbols):
     :rtype: list of str, 0x-hex
     """
     tokens = []
-    chain_spec = CICRegistry.chain_spec
+    chain_spec = TokenTxFactory.chain_spec
     for token_symbol in token_symbols:
         token = CICRegistry.get_token(chain_spec, token_symbol)
         tokens.append({
@@ -322,7 +334,7 @@ def otx_cache_transfer(
         ):
 
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, TokenTxFactory.chain_spec.chain_id())
     (txc, cache_id) = cache_transfer_data(tx_hash_hex, tx)
     return txc
 
@@ -360,7 +372,7 @@ def otx_cache_approve(
         ):
 
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, TokenTxFactory.chain_spec.chain_id())
     (txc, cache_id) = cache_approve_data(tx_hash_hex, tx)
     return txc
 
