@@ -11,9 +11,9 @@ from .rpc import RpcClient
 from cic_eth.db import Otx, SessionBase
 from cic_eth.db.models.tx import TxCache
 from cic_eth.error import PermanentTxError, TemporaryTxError
-from cic_registry import CICRegistry
 from cic_registry import zero_address
 from cic_eth.queue.tx import create as queue_create
+from cic_eth.eth.factory import TxFactory
 from cic_eth.queue.tx import get_tx
 from cic_eth.error import OutOfGasError
 from cic_eth.eth.util import unpack_signed_raw_tx
@@ -251,7 +251,7 @@ def refill_gas(self, recipient_address):
                 'to': recipient_address,
                 'gas': gas_limit,
                 'gasPrice': gas_price,
-                'chainId': CICRegistry.chain_spec.chain_id(),
+                'chainId': TxFactory.chain_spec.chain_id(),
                 'nonce': nonce,
                 'value': refill_amount,
                 'data': '',
@@ -339,7 +339,7 @@ def resend_with_higher_gas(self, txold_hash_hex, gas=None, default_factor=1.1):
     session.close()
 
     tx_signed_raw_bytes = bytes.fromhex(otx.signed_tx[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, TxFactory.chain_spec.chain_id())
     logg.debug('otx {} {}'.format(tx, otx.signed_tx))
 
     queue = self.request.delivery_info['routing_key']
@@ -371,12 +371,16 @@ def resend_with_higher_gas(self, txold_hash_hex, gas=None, default_factor=1.1):
 
 
 @celery_app.task(bind=True)
-def resume_tx(self, txpending_hash_hex):
+def resume_tx(self, txpending_hash_hex, chain_id):
     session = SessionBase.create_session()
-    tx_signed_raw_hex = session.query(Otx.signed_tx).filter(Otx.tx_hash==txold_hash_hex).first()
+    r = session.query(Otx.signed_tx).filter(Otx.tx_hash==txpending_hash_hex).first()
     session.close()
-    if otx == None:
-        raise NotLocalTxError(txold_hash_hex)
+    if r == None:
+        raise NotLocalTxError(txpending_hash_hex)
+
+    tx_signed_raw_hex = r[0]
+    tx_signed_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
+    tx = unpack_signed_raw_tx(tx_signed_bytes, chain_id)
 
     queue = self.request.delivery_info['routing_key']
 
@@ -399,7 +403,7 @@ def otx_cache_parse_tx(
         ):
 
     tx_signed_raw_bytes = bytes.fromhex(tx_signed_raw_hex[2:])
-    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, CICRegistry.chain_spec.chain_id())
+    tx = unpack_signed_raw_tx(tx_signed_raw_bytes, TxFactory.chain_spec.chain_id())
     (txc, cache_id) = cache_gas_refill_data(tx_hash_hex, tx)
     return txc
 
