@@ -16,8 +16,10 @@ def unpack_signed_raw_tx(tx_raw_bytes, chain_id):
 
     for i in d:
         logg.debug('decoded {} chainid {}'.format(i.hex(), chain_id))
-    v = int.from_bytes(d[6], 'big')
-    vb = v - (chain_id * 2) - 35
+    vb = chain_id
+    if chain_id != 0:
+        v = int.from_bytes(d[6], 'big')
+        vb = v - (chain_id * 2) - 35
     s = b''.join([d[7], d[8], bytes([vb])])
     so = KeyAPI.Signature(signature_bytes=s)
     
