@@ -19,7 +19,6 @@ from sqlalchemy import func
 # local imports
 from cic_registry import CICRegistry
 from cic_eth.db.models.otx import Otx
-#from cic_eth.db.models.tx import Tx
 from cic_eth.db.models.tx import TxCache
 from cic_eth.db import SessionBase
 from cic_eth.db.enum import StatusEnum
@@ -186,6 +185,30 @@ def set_fubar(tx_hash):
     session.close()
 
 
+@celery_app.task()
+def set_retry(tx_hash):
+    """Used to set the status when an unexpected error occurs.
+
+    Will set the state to FUBAR
+
+    :param tx_hash: Transaction hash of record to modify
+    :type tx_hash: str, 0x-hex
+    :raises NotLocalTxError: If transaction not found in queue.
+    """
+
+    session = SessionBase.create_session()
+    o = session.query(Otx).filter(Otx.tx_hash==tx_hash).first()
+    if o == None:
+        session.close()
+        raise NotLocalTxError('queue does not contain tx hash {}'.format(tx_hash))
+    session.flush()
+
+    o.retry()
+    session.add(o)
+    session.commit()
+    session.close()
+
+
 @celery_app.task()
 def set_waitforgas(tx_hash):
     """Used to set the status when a transaction must be deferred due to gas refill
@@ -274,7 +297,7 @@ def get_tx(tx_hash):
 
 
 # TODO: move query to model
-def get_paused_txs(status=None, recipient=None):
+def get_paused_txs(status=None, recipient=None, chain_id=0):
     session = SessionBase.create_session()
     q = session.query(Otx)
 
@@ -294,7 +317,7 @@ def get_paused_txs(status=None, recipient=None):
 
     for r in q.all():
         tx_signed_bytes = bytes.fromhex(r.signed_tx[2:])
-        tx = unpack_signed_raw_tx(tx_signed_bytes, int(CICRegistry.chain_spec.chain_id()))
+        tx = unpack_signed_raw_tx(tx_signed_bytes, chain_id)
         if recipient == None or tx['to'] == recipient:
             #gas += tx['gas'] * tx['gasPrice']
             txs[r.tx_hash] = r.signed_tx
@@ -304,8 +327,25 @@ def get_paused_txs(status=None, recipient=None):
     return txs
 
 
+def get_status_tx(status, before=None, limit=0):
+    txs = {}
+    session = SessionBase.create_session()
+    q = session.query(Otx)
+    q = q.join(TxCache)
+    q = q.filter(TxCache.date_updated<before)
+    q = q.filter(Otx.status==status)
+    i = 0
+    for o in q.all():
+        if limit > 0 and i == limit:
+            break
+        txs[o.tx_hash] = o.signed_tx
+        i += 1
+    session.close()
+    return txs
+
+
 # TODO: move query to model
-def get_upcoming_tx(status=StatusEnum.PENDING, recipient=None, since=None):
+def get_upcoming_tx(status=StatusEnum.PENDING, recipient=None, before=None, chain_id=0):
     session = SessionBase.create_session()
     q = session.query(
             TxCache.sender,
@@ -331,8 +371,8 @@ def get_upcoming_tx(status=StatusEnum.PENDING, recipient=None, since=None):
         q = q.filter(TxCache.sender==r.sender)
         q = q.filter(Otx.nonce==r.nonce)
 
-        if since != None:
-            q = q.filter(TxCache.date_checked<since)
+        if before != None:
+            q = q.filter(TxCache.date_checked<before)
 
         o = q.first()
 
@@ -340,7 +380,7 @@ def get_upcoming_tx(status=StatusEnum.PENDING, recipient=None, since=None):
             continue
 
         tx_signed_bytes = bytes.fromhex(o.signed_tx[2:])
-        tx = unpack_signed_raw_tx(tx_signed_bytes, int(CICRegistry.chain_spec.chain_id()))
+        tx = unpack_signed_raw_tx(tx_signed_bytes, chain_id)
         #if recipient == None or tx['to'] == recipient:
             #gas += tx['gas'] * tx['gasPrice']
         txs[o.tx_hash] = o.signed_tx
@@ -358,3 +398,29 @@ def get_upcoming_tx(status=StatusEnum.PENDING, recipient=None, since=None):
 
     return txs
 
+
+@celery_app.task()
+def get_account_tx(address, as_sender=True, as_recipient=True, recipients=None):
+
+    if not as_sender and not as_recipient:
+        raise ValueError('at least one of as_sender and as_recipient must be True')
+    txs = {}
+
+    session = SessionBase.create_session()
+    q = session.query(Otx)
+    q = q.join(TxCache)
+    if as_sender and as_recipient:
+        q = q.filter(or_(TxCache.sender==address, TxCache.recipient==address))
+    elif as_sender:
+        q = q.filter(TxCache.sender==address)
+    else:
+        q = q.filter(TxCache.recipient==address)
+    q = q.order_by(Otx.nonce.asc()) 
+    logg.debug('q {}'.format(q))
+
+    results = q.all()
+    for r in results:
+        txs[r.tx_hash] = r.signed_tx
+    session.close()
+
+    return txs
