@@ -115,7 +115,7 @@ def loop(w3, interval):
             txs = {}
             typ = StatusEnum.WAITFORGAS
             gas_tx_date_offset = datetime.datetime.now() - datetime.timedelta(seconds=1)
-            utxs = get_upcoming_tx(typ, since=gas_tx_date_offset)
+            utxs = get_upcoming_tx(typ, before=gas_tx_date_offset)
             for k in utxs.keys():
                 txs[k] = utxs[k]
             if len(txs.keys()) == 0:
@@ -141,8 +141,8 @@ def loop(w3, interval):
 def main(): 
     c = RpcClient()
 
-    CICRegistry.set_data_dir(config.get('ETH_ABI_DIR'))
-    CICRegistry.finalize(c.w3, config.get('CIC_REGISTRY_ADDRESS'))
+    CICRegistry.init(c.w3, config.get('CIC_REGISTRY_ADDRESS'))
+    CICRegistry.add_path(config.get('ETH_ABI_DIR'))
 
     try:
         loop(c.w3, float(config.get('DISPATCHER_LOOP_INTERVAL')))
