@@ -13,11 +13,14 @@ import celery
 import rlp
 import web3
 from web3 import HTTPProvider, WebsocketProvider
+from cic_registry import CICRegistry
+from cic_registry.chain import ChainSpec
+from cic_registry.chain import ChainRegistry
+from cic_registry.error import UnknownContractError
+from cic_bancor.bancor import BancorRegistryClient
 
 # local imports
 import cic_eth
-from cic_registry import CICRegistry
-from cic_registry.bancor import BancorRegistryClient
 from cic_eth.eth import RpcClient
 from cic_eth.db import SessionBase
 from cic_eth.db import OtxSync
@@ -33,6 +36,9 @@ from cic_eth.db.error import UnknownConvertError
 from cic_eth.eth.util import unpack_signed_raw_tx
 from cic_eth.eth.task import create_check_gas_and_send_task
 from cic_eth.sync.backend import SyncerBackend
+from cic_eth.eth.token import unpack_transfer
+from cic_eth.eth.token import unpack_transferfrom
+from cic_eth.eth.faucet import unpack_giveto
 
 logging.basicConfig(level=logging.WARNING)
 logg = logging.getLogger()
@@ -42,11 +48,11 @@ logging.getLogger('web3.providers.WebsocketProvider').setLevel(logging.CRITICAL)
 logging.getLogger('web3.providers.HTTPProvider').setLevel(logging.CRITICAL)
 
 
-
 config_dir = os.path.join('/usr/local/etc/cic-eth')
 
 argparser = argparse.ArgumentParser(description='daemon that monitors transactions in new blocks')
 argparser.add_argument('-c', type=str, default=config_dir, help='config root to use')
+argparser.add_argument('-i', '--chain-spec', type=str, dest='i', help='chain spec')
 argparser.add_argument('--abi-dir', dest='abi_dir', type=str, help='Directory containing bytecode and abi')
 argparser.add_argument('--env-prefix', default=os.environ.get('CONFINI_ENV_PREFIX'), dest='env_prefix', type=str, help='environment prefix for variables to overwrite configuration')
 argparser.add_argument('-q', type=str, default='cic-eth', help='celery queue to submit transaction tasks to')
@@ -67,6 +73,7 @@ config.process()
 # override args
 args_override = {
         'ETH_ABI_DIR': getattr(args, 'abi_dir'),
+        'CIC_CHAIN_SPEC': getattr(args, 'i'),
         }
 config.dict_override(args_override, 'cli flag')
 config.censor('PASSWORD', 'DATABASE')
@@ -80,10 +87,25 @@ queue = args.q
 dsn = dsn_from_config(config)
 SessionBase.connect(dsn)
 
+# TODO: There is too much code in this file, split it up
+
+transfer_callbacks = []
+for cb in config.get('TASKS_TRANSFER_CALLBACKS', '').split(','):
+    task_split = cb.split(':')
+    task_queue = queue
+    if len(task_split) > 1:
+        task_queue = task_split[0]
+    task_pair = (task_split[1], task_queue)
+    transfer_callbacks.append(task_pair)
+
+
 # TODO: move to contract registry
 __convert_log_hash = '0x7154b38b5dd31bb3122436a96d4e09aba5b323ae1fd580025fab55074334c095' # keccak256(Conversion(address,address,address,uint256,uint256,address)
 __account_registry_add_log_hash = '0x5ed3bdd47b9af629827a8d129aa39c870b10c03f0153fe9ddb8e84b665061acd' # keccak256(AccountAdded(address,uint256))
 
+__transfer_method_signature = '0xa9059cbb' # keccak256(transfer(address,uint256))
+__transferfrom_method_signature = '0x23b872dd' # keccak256(transferFrom(address,address,uint256))
+__giveto_method_signature = '0x63e4bff4' # keccak256(giveTo(address))
 
 # TODO: move to bancor package
 def parse_convert_log(w3, entry):
@@ -181,6 +203,71 @@ def tx_filter(w3, tx, rcpt):
     return t
 
 
+def get_token_symbol(w3, address):
+    #token = CICRegistry.get_address(CICRegistry.chain_spec, tx['to'])
+    logg.warning('token verification missing')
+    c = w3.eth.contract(abi=CICRegistry.abi('ERC20'), address=address)
+    return c.functions.symbol().call()
+
+
+def callbacks_filter(w3, tx, rcpt):
+    transfer_data = None
+    if len(tx.input) < 10:
+        logg.debug('callbacks filter data length not sufficient for method signature in tx {}, skipping'.format(tx['hash']))
+        return
+
+    logg.debug('checking callbacks filter input {}'.format(tx.input[:10]))
+
+    transfer_type = 'transfer'
+    method_signature = tx.input[:10]
+    if method_signature == __transfer_method_signature:
+        transfer_data = unpack_transfer(tx.input)
+        transfer_data['from'] = tx['from']
+        transfer_data['token_address'] = tx['to']
+    elif method_signature == __transferfrom_method_signature:
+        transfer_type = 'transferfrom'
+        transfer_data = unpack_transferfrom(tx.input)
+        transfer_data['token_address'] = tx['to']
+    elif method_signature == __giveto_method_signature:
+        transfer_type = 'tokengift'
+        transfer_data = unpack_giveto(tx.input)
+        for l in rcpt.logs:
+            if l.topics[0].hex() == '0x45c201a59ac545000ead84f30b2db67da23353aa1d58ac522c48505412143ffa':
+                transfer_data['amount'] = web3.Web3.toInt(hexstr=l.data)
+                token_address_bytes = l.topics[2][32-20:]
+                transfer_data['token_address'] = web3.Web3.toChecksumAddress(token_address_bytes.hex())
+                transfer_data['from'] = rcpt.to
+
+    if transfer_data != None:
+        for tc in transfer_callbacks:
+            token_symbol = None
+            try:
+                logg.debug('checking token {}'.format(transfer_data['token_address']))
+                token_symbol = get_token_symbol(w3, transfer_data['token_address'])
+                logg.debug('calling transfer callback {}:{} for tx {}'.format(tc[1], tc[0], tx['hash']))
+            except UnknownContractError:
+                logg.debug('callback filter {}:{} skipping "transfer" method on unknown contract {} tx {}'.format(tc[1], tc[0], transfer_data['to'], tx.hash.hex()))
+                continue
+            result = {
+                'hash': tx.hash.hex(),
+                'sender': transfer_data['from'],
+                'recipient': transfer_data['to'],
+                'value': transfer_data['amount'],
+                'token': transfer_data['token_address'],
+                'token_symbol': token_symbol,
+                    }
+            s = celery.signature(
+                tc[0],
+                [
+                    result,
+                    transfer_type,
+                    int(rcpt.status == 1),
+                ],
+                queue=tc[1],
+                )
+            s.apply_async()
+
+
 class GasFilter:
 
     def __init__(self, gas_provider):
@@ -235,29 +322,31 @@ RpcClient.set_constructor(web3_constructor)
 def main(): 
     c = RpcClient()
 
-    CICRegistry.set_data_dir(config.get('ETH_ABI_DIR'))
-    CICRegistry.finalize(c.w3, config.get('CIC_REGISTRY_ADDRESS'))
+    CICRegistry.init(c.w3, config.get('CIC_REGISTRY_ADDRESS'))
+    CICRegistry.add_path(config.get('ETH_ABI_DIR'))
+    chain_spec = ChainSpec.from_chain_str(config.get('CIC_CHAIN_SPEC'))
+    chain_registry = ChainRegistry(chain_spec)
+    CICRegistry.add_chain_registry(chain_registry)
 
-    chain = str(CICRegistry.chain_spec)
-    
     syncers = []
     block_offset = c.w3.eth.blockNumber
+    chain = str(chain_spec)
 
     if SyncerBackend.first(chain):
         from cic_eth.sync.history import HistorySyncer
         backend = SyncerBackend.initial(chain, block_offset)
-        syncer = HistorySyncer(c, backend) 
+        syncer = HistorySyncer(backend) 
         syncers.append(syncer)
 
     if args.mode == 'head':
         from cic_eth.sync.head import HeadSyncer
         block_sync = SyncerBackend.live(chain, block_offset+1)
-        syncers.append(HeadSyncer(c, block_sync))
+        syncers.append(HeadSyncer(block_sync))
     elif args.mode == 'history':
         from cic_eth.sync.history import HistorySyncer
         backends = SyncerBackend.resume(chain, block_offset+1)
         for backend in backends:
-            syncers.append(HistorySyncer(c, backend))
+            syncers.append(HistorySyncer(backend))
         if len(syncers) == 0:
             logg.info('found no unsynced history. terminating')
             sys.exit(0)
@@ -265,8 +354,8 @@ def main():
         sys.stderr.write("unknown mode '{}'\n".format(args.mode))
         sys.exit(1)
 
-    bancor_registry_contract = CICRegistry.get_contract(CICRegistry.bancor_chain_spec, 'BancorRegistry')
-    bancor_chain_registry = CICRegistry.get_chain_registry(CICRegistry.bancor_chain_spec)
+    bancor_registry_contract = CICRegistry.get_contract(chain_spec, 'BancorRegistry', interface='Registry')
+    bancor_chain_registry = CICRegistry.get_chain_registry(chain_spec)
     bancor_registry = BancorRegistryClient(c.w3, bancor_chain_registry, config.get('ETH_ABI_DIR'))
     bancor_registry.load() 
  
@@ -276,6 +365,7 @@ def main():
         gas_filter = GasFilter(c.gas_provider()).filter
         syncer.filter.append(gas_filter)
         syncer.filter.append(registration_filter)
+        syncer.filter.append(callbacks_filter)
         # TODO: the two following filter functions break the filter loop if return uuid. Pro: less code executed. Con: Possibly unintuitive flow break
         syncer.filter.append(tx_filter)
         syncer.filter.append(convert_filter)
