@@ -14,7 +14,10 @@ from web3 import HTTPProvider, WebsocketProvider
 
 # local imports
 from cic_registry.registry import CICRegistry
-from cic_registry.bancor import BancorRegistryClient
+from cic_registry.registry import ChainRegistry
+from cic_registry.registry import ChainSpec
+
+from cic_bancor.bancor import BancorRegistryClient
 from cic_eth.eth import bancor
 from cic_eth.eth import token
 from cic_eth.eth import tx
@@ -36,6 +39,7 @@ argparser = argparse.ArgumentParser()
 argparser.add_argument('-c', type=str, default=config_dir, help='config file')
 argparser.add_argument('-q', type=str, default='cic-eth', help='queue name for worker tasks')
 argparser.add_argument('--abi-dir', dest='abi_dir', type=str, help='Directory containing bytecode and abi')
+argparser.add_argument('-i', '--chain-spec', dest='i', type=str, help='chain spec')
 argparser.add_argument('--env-prefix', default=os.environ.get('CONFINI_ENV_PREFIX'), dest='env_prefix', type=str, help='environment prefix for variables to overwrite configuration')
 argparser.add_argument('-v', action='store_true', help='be verbose')
 argparser.add_argument('-vv', action='store_true', help='be more verbose')
@@ -51,6 +55,7 @@ config.process()
 # override args
 args_override = {
         'ETH_ABI_DIR': getattr(args, 'abi_dir'),
+        'CIC_CHAIN_SPEC': getattr(args, 'i'),
         }
 config.dict_override(args_override, 'cli flag')
 config.censor('PASSWORD', 'DATABASE')
@@ -151,17 +156,15 @@ def main():
     if config.get('SSL_CA_FILE') != '':
         Callback.ssl_ca_file = config.get('SSL_CA_FILE')
 
-    #GasOracle.set_provider_address(config.get('ETH_GAS_PROVIDER_ADDRESS'))
-    # cannot use websockets with celery
-    # https://stackoverflow.com/questions/59223741/calling-api-within-celery-task-never-returns
-    #w3 = rpc.connect(config.get('ETH_PROVIDER'), config.get('SIGNER_SOCKET_PATH'))
-
     c = RpcClient()
-    CICRegistry.set_data_dir(config.get('ETH_ABI_DIR'))
-    CICRegistry.finalize(c.w3, config.get('CIC_REGISTRY_ADDRESS'))
-    bancor_registry_contract = CICRegistry.get_contract(CICRegistry.bancor_chain_spec, 'BancorRegistry')
-    bancor_chain_registry = CICRegistry.get_chain_registry(CICRegistry.bancor_chain_spec)
-    #bancor_registry = BancorRegistry(c.w3, bancor_chain_registry, bancor_registry_contract.address(), config.get('ETH_ABI_DIR'))
+    CICRegistry.init(c.w3, config.get('CIC_REGISTRY_ADDRESS'))
+    CICRegistry.add_path(config.get('ETH_ABI_DIR'))
+    chain_spec = ChainSpec.from_chain_str(config.get('CIC_CHAIN_SPEC'))
+    chain_registry = ChainRegistry(chain_spec)
+    CICRegistry.add_chain_registry(chain_registry)
+
+    bancor_registry_contract = CICRegistry.get_contract(chain_spec, 'BancorRegistry', interface='Registry')
+    bancor_chain_registry = CICRegistry.get_chain_registry(chain_spec)
     bancor_registry = BancorRegistryClient(c.w3, bancor_chain_registry, config.get('ETH_ABI_DIR'))
     bancor_registry.load(True)
     current_app.worker_main(argv)
