@@ -46,13 +46,13 @@ default_config_dir = os.path.join('/usr/local/etc/cic-eth')
 argparser = argparse.ArgumentParser()
 argparser.add_argument('-p', '--provider', dest='p', default='http://localhost:8545', type=str, help='Web3 provider url (http only)')
 argparser.add_argument('-r', '--registry-address', type=str, help='CIC registry address')
-argparser.add_argument('-f', '--output-format', dest='f', default='terminal', type=str, help='Output format')
+argparser.add_argument('-f', '--format', dest='f', default='terminal', type=str, help='Output format')
 argparser.add_argument('-c', type=str, default=default_config_dir, help='config root to use')
 argparser.add_argument('-q', type=str, default='cic-eth', help='celery queue to submit transaction tasks to')
 argparser.add_argument('--env-prefix', default=os.environ.get('CONFINI_ENV_PREFIX'), dest='env_prefix', type=str, help='environment prefix for variables to overwrite configuration')
 argparser.add_argument('-v', action='store_true', help='Be verbose')
 argparser.add_argument('-vv', help='be more verbose', action='store_true')
-argparser.add_argument('tx_hash_or_tx_raw', type=str, help='Transaction')
+argparser.add_argument('query', type=str, help='Transaction')
 args = argparser.parse_args()
 
 if args.v == True:
@@ -64,6 +64,9 @@ config_dir = os.path.join(args.c)
 os.makedirs(config_dir, 0o777, True)
 config = confini.Config(config_dir, args.env_prefix)
 config.process()
+args_override = {
+        'ETH_PROVIDER': getattr(args, 'p'),
+        }
 # override args
 config.censor('PASSWORD', 'DATABASE')
 config.censor('PASSWORD', 'SSL')
@@ -106,18 +109,21 @@ elif args.f[:1] != 't':
 
 def main():
     # TODO: add custom contract translation 
-    tx = None
-    if len(args.tx_hash_or_tx_raw) > 66:
-        tx = admin_api.tx(tx_raw=args.tx_hash_or_tx_raw)
+    txs  = None
+    if len(args.query) > 66:
+        txs = [admin_api.tx(tx_raw=args.query)]
+    elif len(args.query) > 42:
+        txs = [admin_api.tx(tx_hash=args.query)]
     else:
-        tx = admin_api.tx(tx_hash=args.tx_hash_or_tx_raw)
+        txs = admin_api.account(args.query)
 
     if fmt == 'json':
-        sys.stdout.write(json.dumps(tx))
+        sys.stdout.write(json.dumps(txs))
     else:
-        for k in tx.keys():
-            sys.stdout.write('{}: {}\n'.format(k, tx[k]))
-
+        for tx in txs:
+            print()
+            for k in tx.keys():
+                sys.stdout.write('{}: {}\n'.format(k, tx[k]))
 
 if __name__ == '__main__':
     main()
