@@ -1,5 +1,6 @@
 # standard imports
 import logging
+import time
 
 # third-party imports
 import celery
@@ -7,6 +8,7 @@ import celery
 # local impotes
 from .base import Syncer
 from cic_eth.queue.tx import set_final_status
+from cic_eth.eth import RpcClient
 
 app = celery.current_app
 logg = logging.getLogger()
@@ -21,28 +23,26 @@ class MinedSyncer(Syncer):
     :type block_hash_hex: str, 0x-hash
     """
 
-    def __init__(self, rpc, bc_cache):
+    def __init__(self, bc_cache):
         super(MinedSyncer, self).__init__(bc_cache)
-        self.rpc = rpc
-        self.w3 = rpc.w3
         self.block_offset = 0
         self.tx_offset = 0
 
 
-    def process(self, w3, block_hash_hex):
-        b = w3.eth.getBlock(block_hash_hex)
-        c = w3.eth.getBlockTransactionCount(block_hash_hex)
+    def process(self, w3, ref):
+        b = w3.eth.getBlock(ref)
+        c = w3.eth.getBlockTransactionCount(ref)
         s = 0
         if self.block_offset == b.number:
             s = self.tx_offset
 
-        logg.debug('processing {} (blocknumber {}, count {}, offset {})'.format(block_hash_hex, b.number, c, s))
+        logg.debug('processing {} (blocknumber {}, count {}, offset {})'.format(ref, b.number, c, s))
 
         for i in range(s, c):
-            tx = w3.eth.getTransactionByBlock(block_hash_hex, i)
+            tx = w3.eth.getTransactionByBlock(ref, i)
             tx_hash_hex = tx['hash'].hex()
             rcpt = w3.eth.getTransactionReceipt(tx_hash_hex)
-            logg.debug('{}/{} processing tx {} from block {} {}'.format(i+1, c, tx_hash_hex, b.number, block_hash_hex))
+            logg.debug('{}/{} processing tx {} from block {} {}'.format(i+1, c, tx_hash_hex, b.number, ref))
             ours = False
             # TODO: ensure filter loop can complete on graceful shutdown
             for f in self.filter:
@@ -70,3 +70,25 @@ class MinedSyncer(Syncer):
             self.bc_cache.set(b.number+1, 0)
         return b['number']
 
+
+
+    def loop(self, interval):
+        """Loop running until the "running" property of Syncer is set to False.
+
+        Retrieves latest unprocessed blocks and processes them.
+
+        :param interval: Delay in seconds until next attempt if no new blocks are found.
+        :type interval: int
+        """
+        while self.running and Syncer.running_global:
+            self.bc_cache.connect()
+            c = RpcClient()
+            logg.debug('loop execute')
+            e = self.get(c.w3)
+            logg.debug('got blocks {}'.format(e))
+            for block in e:
+                block_number = self.process(c.w3, block.hex())
+                logg.info('processed block {} {}'.format(block_number, block.hex()))
+            self.bc_cache.disconnect()
+            time.sleep(interval)
+        logg.info("Syncer no longer set to run, gracefully exiting")
